@@ -31,19 +31,17 @@
  */
 
 /////////////////////// Std lib includes
-#include <MassDataCborBaseHandler.hpp>
 #include <set>
 #include <cassert>
 
 
 /////////////////////// Qt includes
-#include <IsotopicData.hpp>
-#include <IsotopicDataBaseHandler.hpp>
 #include <QDebug>
 #include <QFileDialog>
 #include <QSettings>
 #include <QAbstractItemView>
 #include <QHeaderView>
+#include <QMessageBox>
 
 
 /////////////////////// IsoSpec
@@ -65,27 +63,31 @@
 // elem_table_log_probability[ISOSPEC_NUMBER_OF_ISOTOPIC_ENTRIES];
 
 
+/////////////////////// PAPPSO includes
+
+
+
 /////////////////////// libmass includes
 #include <libmass/globals.hpp>
 #include <libmass/Isotope.hpp>
+#include <libmass/IsotopicData.hpp>
+#include <libmass/IsotopicDataBaseHandler.hpp>
 #include <libmass/IsotopicDataLibraryHandler.hpp>
+#include <libmass/MassDataCborBaseHandler.hpp>
 #include <libmass/IsotopicDataUserConfigHandler.hpp>
 #include <libmass/IsotopicDataManualConfigHandler.hpp>
 #include <libmass/PeakCentroid.hpp>
 #include <libmass/IsotopicClusterGenerator.hpp>
-
-#include <libmassgui/ColorSelector.hpp>
+#include <libmass/MassDataCborBaseHandler.hpp>
 
 
 /////////////////////// Local includes
 #include "IsotopicDataTableViewModel.hpp"
 #include "IsotopicClusterGeneratorDlg.hpp"
-#include "IsotopicClusterShaperDlg.hpp"
-
+#include "ColorSelector.hpp"
 #include "IsotopicClusterGeneratorDlg.hpp"
-
-// For the element data groupbox widgets that are packed dynamically
-#include "pappsomspp/trace/trace.h"
+#include "IsotopicClusterShaperDlg.hpp"
+// For the element data groupbox widgets that are packed dynamicallyx
 #include "ui_ElementGroupBoxWidget.h"
 
 
@@ -102,6 +104,7 @@ IsotopicClusterGeneratorDlg::IsotopicClusterGeneratorDlg(
   const QString &description)
   : QDialog(program_window_p),
     m_applicationName{applicationName},
+    m_windowDescription(description),
     mp_programWindow(program_window_p)
 {
   if(!program_window_p)
@@ -533,8 +536,13 @@ IsotopicClusterGeneratorDlg::traceColorPushButtonClicked()
 void
 IsotopicClusterGeneratorDlg::toIsotopicClusterShaper()
 {
-  // We need to get the text from the output text edit widget and send that to
-  // the input widget of the PeakShaperDlg.
+  // The calculation result is set into a vector that might contain more than
+  // one pair (charge,cluster). In the present case, the dialog window allows
+  // for only one elemental composition to perform the calculation, so we just
+  // need to take the first (and in fact there must be only one) charge,cluster
+  // pair from the vector. We get the trace and we sent that to the peak shaper
+  // dialog. We also want to set the color that might have been set by the user
+  // for plotting the trace.
 
   const std::vector<libmass::IsotopicClusterChargePair>
     &isotopic_cluster_charge_pairs =
@@ -565,7 +573,7 @@ IsotopicClusterGeneratorDlg::toIsotopicClusterShaper()
 
   // In that shaper dialog, when the computation has finished, the user may ask
   // to display the spectrum. So the dialog window emits the signal below. We
-  // want to relay that signal to the other listeners, typicall the program
+  // want to relay that signal to the other listeners, typically the program
   // window, so that they can take action. If the program window is of
   // massXpert, they'll have to craft a CBOR-encoded packet and serve it so that
   // any listening program can display it (typically mineXpert). If that program
@@ -1300,7 +1308,19 @@ IsotopicClusterGeneratorDlg::runUserConfig()
   // The correct isotopic data handler is used to perform the preparation of the
   // arrays needed by IsoSpec.
 
-  assert(msp_isotopicDataUserConfig->size());
+  if(!msp_isotopicDataUserConfig->size())
+    {
+      QMessageBox::warning(
+        this,
+        QString("%1 - %2").arg(m_applicationName).arg(m_windowDescription),
+        "Please, load a user-defined isotopic data table.\n"
+        "Alternatively, use the static data table view (other tab)\n"
+        "and modify it in place, then save it to disk.\n"
+        "Finally load these isotopic data from file into this table view.",
+        QMessageBox::Ok);
+
+      return false;
+    }
 
   if(!checkParameters())
     {
@@ -1540,7 +1560,7 @@ IsotopicClusterGeneratorDlg::initializeIsoSpecManualConfigurationWidgets(
   using SymbolCountMap      = std::map<QString, int>;
   const SymbolCountMap &map = config_handler.getSymbolCountMap();
 
-  //qDebug() << "The symbol/count map has this count of items:" << map.size();
+  // qDebug() << "The symbol/count map has this count of items:" << map.size();
 
   // Get all the unqiue isotope symbols that were set the isotopic data object.
   // Remember that in the manual config context, only isotopes from the user's
@@ -1556,7 +1576,7 @@ IsotopicClusterGeneratorDlg::initializeIsoSpecManualConfigurationWidgets(
       // This call throws if the symbol is not there.
       int symbol_count = map.at(symbol);
 
-      //qDebug() << "For symbol" << symbol << "the count is:" << symbol_count;
+      // qDebug() << "For symbol" << symbol << "the count is:" << symbol_count;
 
       // This is where we contruct a new elementGroupBox
       QGroupBox *elementGroupBox = addElementSkeletonGroupBox();
@@ -1670,7 +1690,7 @@ IsotopicClusterGeneratorDlg::reportResults()
 
   QString results = m_clusterGenerator.clustersToString();
 
-  //qDebug().noquote() << results;
+  // qDebug().noquote() << results;
 
   m_ui.isoSpecTabWidget->setCurrentWidget(m_ui.isoSpecTabWidgetResultsTab);
   m_ui.isoSpecOutputDataPlainTextEdit->setPlainText(results);
