@@ -31,15 +31,20 @@
  */
 
 
+/////////////////////// Std lib includes
+#include <memory>
+
+
 /////////////////////// Qt includes
 
 
+/////////////////////// libmass includes
+#include <libmass/Isotope.hpp>
+
+
 /////////////////////// Local includes
 #include "IsotopicDataTableViewModel.hpp"
-#include <Isotope.hpp>
-#include <libmass/Isotope.hpp>
-#include <IsotopicClusterGeneratorDlg.hpp>
-#include <memory>
+#include "IsotopicClusterGeneratorDlg.hpp"
 
 
 namespace msxps
