@@ -32,19 +32,26 @@
 
 
 /////////////////////// Qt includes
-#include <IsotopicClusterGeneratorDlg.hpp>
-#include <IsotopicData.hpp>
 #include <QtGui>
 #include <QtXml>
 #include <QtGlobal>
 #include <QMessageBox>
 
+
+/////////////////////// libmass includes
+#include <libmass/IsotopicData.hpp>
+
+
+/////////////////////// libmassgui includes
+#include <libmass/PolChemDefEntity.hpp>
+#include <libmass/Sequence.hpp>
+#include <libmassgui/IsotopicClusterGeneratorDlg.hpp>
+
+
 /////////////////////// Local includes
 #include "CalculatorWnd.hpp"
 #include "ProgramWindow.hpp"
 #include "CalculatorRecorderDlg.hpp"
-#include "libmass/PolChemDefEntity.hpp"
-#include "libmass/Sequence.hpp"
 #include "MzCalculationDlg.hpp"
 #include "../nongui/ConfigSetting.hpp"
 
@@ -102,6 +109,15 @@ CalculatorWnd::CalculatorWnd(ProgramWindow *parent,
     }
 
 
+
+    QString formula_string = "-H2O";
+  libmass::Formula formula(formula_string);
+
+  formula.validate(msp_polChemDef->getIsotopicDataCstSPtr());
+
+
+
+
   // If the strings mono and avg are non null, then set their values
   // to the seed masses.
 
@@ -288,7 +304,7 @@ CalculatorWnd::initialize()
           this,
           &CalculatorWnd::isotopicPatternCalculation);
 
-  m_ui.formulaSpinBox->setRange(-1000000000, 1000000000);
+  m_ui.formulaDoubleSpinBox->setRange(-1000000000, 1000000000);
   m_ui.monomerSpinBox->setRange(-1000000000, 1000000000);
   m_ui.modifSpinBox->setRange(-1000000000, 1000000000);
   m_ui.sequenceSpinBox->setRange(-1000000000, 1000000000);
@@ -870,6 +886,9 @@ CalculatorWnd::clearResult()
 void
 CalculatorWnd::apply(const QString &formula)
 {
+  // This function might be called from the chempad's buttons with the
+  // corresponding formula.
+
   QString mass;
 
   bool calculationPerformed = false;
@@ -1196,11 +1215,13 @@ CalculatorWnd::isotopicPatternCalculation()
 
 
 int
-CalculatorWnd::accountFormula(const QString &formula, int times)
+CalculatorWnd::accountFormula(const QString &formula, double times)
 {
-  QString text;
-  int count;
+  QString formula_text;
 
+  // Note how count is a double and not an int. This has
+  // implications below.
+  double count;
 
   // This function might be called via the chemical pad, and thus has
   // to test existence of atom definitions. The 'apply()' function
@@ -1224,9 +1245,9 @@ CalculatorWnd::accountFormula(const QString &formula, int times)
     {
       // No string passed as parameter, get the formula from its
       // corresponding comboBox.
-      text = m_ui.formulaComboBox->currentText();
+      formula_text = m_ui.formulaComboBox->currentText();
 
-      if(text.isEmpty())
+      if(formula_text.isEmpty())
         return 0;
 
       // We want to remove spaces from the text. This is useful so
@@ -1234,43 +1255,59 @@ CalculatorWnd::accountFormula(const QString &formula, int times)
       // re-reading the formula text (useful when entering complex
       // formula for saccharides, for example, in my own
       // experience).
-      text.remove(QRegularExpression("\\s+"));
+      formula_text.remove(QRegularExpression("\\s+"));
 
       // Note that we might also have a title for the formula, which
       // is enclosed between `"' at the beginning of the line.
 
-      text.remove(QRegularExpression("\".*\""));
+      formula_text.remove(QRegularExpression("\".*\""));
 
       // How many times should the formula be accounted for?
-      count = m_ui.formulaSpinBox->value();
+      count = m_ui.formulaDoubleSpinBox->value();
 
       if(count == 0)
         return 0;
     }
   else
     {
-      text = formula;
+      formula_text = formula;
 
       // The times param tells us how many times should the formula be
       // accounted for.
       count = times;
     }
 
-  libmass::Modif fake(msp_polChemDef, "NOT_SET", text);
+  libmass::Modif fake(msp_polChemDef, "NOT_SET", formula_text);
+
+  // First calculate the masses for the formula. These masses
+  // become member data of the Modif.
   if(!fake.calculateMasses())
     return -1;
 
   // Prepare the recorder string.
   QString recorder =
-    tr("Accounting formula: %1(%2 times)... ").arg(text).arg(count);
+    tr("Accounting formula: %1(%2 times)... ").arg(formula_text).arg(count);
 
   // And now append the new recorder string to the recorder textEdit.
   mpa_recorderDlg->record(recorder);
 
-  if(!fake.accountMasses(
-       &m_tempPonderable.rmono(), &m_tempPonderable.ravg(), count))
+  // And now actually compute the compounding of the pre-calculated masses
+  // with the count (which might be a double). Because it might be
+  // a double, we have to first account the masses once, and then
+  // multiply their result.
+
+  double temp_mono = 0;
+  double temp_avg = 0;
+
+  if(!fake.accountMasses(&temp_mono, &temp_avg, 1))
     return -1;
 
+  // Now that we know to what mass corresponds the formula_text,
+  // we can multiply these by the factor that is actually the count.
+
+     m_tempPonderable.rmono() += temp_mono * count;
+     m_tempPonderable.ravg() += temp_avg * count;
+
   return 1;
 }
 
