@@ -32,7 +32,6 @@
 
 
 /////////////////////// Qt includes
-#include <Oligomer.hpp>
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QApplication>
@@ -41,6 +40,7 @@
 
 
 /////////////////////// libmass includes
+#include <libmass/Oligomer.hpp>
 #include <libmass/PolChemDefEntity.hpp>
 #include <libmass/PolChemDef.hpp>
 
@@ -220,6 +220,9 @@ MzCalculationDlg::formulaCheckBoxToggled(bool checked)
 void
 MzCalculationDlg::calculate()
 {
+  libmass::IsotopicDataCstSPtr isotopic_data_csp =
+    mcsp_polChemDef->getIsotopicDataCstSPtr();
+
   // The calculation can be accomplished in two different manners:
   //
   // 1. Starting from a formula, in which case it will be necessary
@@ -238,7 +241,7 @@ MzCalculationDlg::calculate()
   // same dialog window, make sure that the formula is cleared.
   m_formula.clear();
 
-  // The source ionizeRule. Start with this because we'll need right
+  // The source ionizeRule. Start with this because we'll need it right
   // away.
   libmass::IonizeRule srcIonizeRule;
   if(!getSrcIonizeRuleData(&srcIonizeRule))
@@ -261,13 +264,17 @@ MzCalculationDlg::calculate()
           return;
         }
 
-      libmass::Formula formula(text);
+      m_formula.setFormula(text);
 
-      libmass::IsotopicDataCstSPtr isotopic_data_csp =
-        mcsp_polChemDef->getIsotopicDataCstSPtr();
+      // At this point we know we could actually validate the formula,
+      // so do that work with the member formula:
 
+      // We want to validate the formula and in the mean time construct
+      // the list of all the libmass::Isotope objects(first true), and since
+      // the formula is reused we also ensure that that list is reset
+      // (second true).
 
-      if(!formula.validate(isotopic_data_csp, false, true))
+      if(!m_formula.validate(isotopic_data_csp, true, true))
         {
           m_ui.monoLineEdit->setText("0.000");
           m_ui.avgLineEdit->setText("0.000");
@@ -280,23 +287,16 @@ MzCalculationDlg::calculate()
                                  .arg(__LINE__),
                                QMessageBox::Ok);
 
+          // Let the results printing function that we did not
+          // use the formula.
+          m_formula.clear();
+
           return;
         }
 
       // qDebug() << __FILE__ << __LINE__
       //          << "Formula:" << formula.formula() << "validated." ;
 
-      // At this point we know we could actually validate the formula,
-      // so do that work with the member formula:
-
-      // We want to validate the formula and in the mean time construct
-      // the list of all the libmass::Isotope objects(first true), and since
-      // the formula is reused we also ensure that that list is reset
-      // (second true).
-
-      m_formula.setFormula(text);
-      m_formula.validate(isotopic_data_csp, true, true);
-
       // Calculation of the m/z corresponding to the formula requires
       // dividing the mass obtained for the formula by the product
       // (level * unitary) of the ionizerule.
@@ -311,12 +311,19 @@ MzCalculationDlg::calculate()
       if(!m_formula.accountMasses(isotopic_data_csp, &monoMass, &avgMass, 1))
         qFatal("Fatal error at %s@%d.Aborting.", __FILE__, __LINE__);
 
+      // qDebug() << qSetRealNumberPrecision(6)
+      //          << "The masses computed for the formula:" << monoMass << "-"
+      //          << avgMass;
+
       if(charge)
         {
           monoMass /= charge;
           avgMass /= charge;
         }
 
+      // qDebug() << qSetRealNumberPrecision(6) << "Accounting the charge"
+      //          << charge << "they become:" << monoMass << "-" << avgMass;
+
       // Great, now show the data.
 
       QString mz;
@@ -338,8 +345,10 @@ MzCalculationDlg::calculate()
 
   // Whatever the way we get the mon/avg mz values, make a
   // ponderable with them.
-  libmass::Ponderable *ponderable = getSourcePonderable();
-  if(!ponderable)
+
+  std::unique_ptr<libmass::Ponderable> ponderable_up = std::make_unique<libmass::Ponderable>(getSourcePonderable());
+
+  if(!ponderable_up)
     return;
 
   // Create an libmass::Ionizable with empty masses but with proper
@@ -348,15 +357,12 @@ MzCalculationDlg::calculate()
   // boolean.
   libmass::Ionizable ionizable(mcsp_polChemDef,
                                "NOT_SET",
-                               *ponderable,
+                               *ponderable_up,
                                srcIonizeRule,
                                (srcIonizeRule.level() ? true : false));
 
   // At this point the ionizable is fully qualified.
 
-  // No longer in use.
-  delete ponderable;
-
   // Now, because we want to be able to compute elemental
   // compositions for each destination ionization level, we have to
   // kraft an elemental representation of what the molecule
@@ -371,17 +377,12 @@ MzCalculationDlg::calculate()
       // the formula the equivalent atoms of the source ionizerule
       // multiplied by ionization level.
 
-      // qDebug() << __FILE__ << __LINE__
-      //          << "Formula before deionization:" << m_formula.text();
-
-      libmass::IsotopicDataCstSPtr isotopic_data_csp =
-        mcsp_polChemDef->getIsotopicDataCstSPtr();
+      // qDebug() << "Formula before deionization:" << m_formula.toString();
 
       m_formula.accountFormula(
         srcIonizeRule.formula(), isotopic_data_csp, -1 * srcIonizeRule.level());
 
-      // qDebug() << __FILE__ << __LINE__
-      //          << "Formula after deionization:" << m_formula.text();
+      // qDebug() << "Formula after deionization:" << m_formula.toString();
     }
 
   // At this point, the source ionizable is filled with data. And we
@@ -399,12 +400,8 @@ MzCalculationDlg::calculate()
   int startLevel = m_ui.destIonizeRuleStartLevelSpinBox->value();
   int endLevel   = m_ui.destIonizeRuleEndLevelSpinBox->value();
 
-  if(startLevel > endLevel)
-    {
-      int temp   = endLevel;
-      endLevel   = startLevel;
-      startLevel = temp;
-    }
+  // qDebug() << "Start and end levels of ionization:" << startLevel << "->"
+  //          << endLevel;
 
   // Empty the treeView model, so that we can put fresh data in
   // it. Same for the oligomer list!
@@ -413,10 +410,14 @@ MzCalculationDlg::calculate()
 
   m_oligomerList.clear();
 
-  for(int level = startLevel; level < endLevel + 1; ++level)
+  for(int level = std::min(startLevel, endLevel);
+      level < std::max(startLevel, endLevel) + 1;
+      ++level)
     {
       destIonizeRule.setLevel(level);
 
+      // qDebug() << "Now ionizing with rule:" << destIonizeRule.toString();
+
       // Reionize the ionizable.
       if(ionizable.ionize(destIonizeRule) == -1)
         {
@@ -431,6 +432,8 @@ MzCalculationDlg::calculate()
           return;
         }
 
+      // qDebug() << "After ionization, the ionizable:" << ionizable.toString();
+
       libmass::Ionizable *newIonizable = new libmass::Ionizable(ionizable);
 
       mpa_mzTreeViewModel->addIonizable(newIonizable);
@@ -442,17 +445,13 @@ MzCalculationDlg::calculate()
       libmass::OligomerSPtr oligomer_sp = std::make_shared<libmass::Oligomer>(
         ionizable, libmass::CalcOptions(), -1, -1);
 
-      // qDebug() << __FILE__ << __LINE__
-      //          << "oligomer->charge:" << oligomer->charge();
+      // qDebug() << "oligomer->charge:" << oligomer->charge();
 
       // If we are working from a formula, then compute the new
       // formula for the present ionization status and create a
       // property with that new formula to append to the list of
       // properties in the oligomer.
 
-      libmass::IsotopicDataCstSPtr isotopic_data_csp =
-        mcsp_polChemDef->getIsotopicDataCstSPtr();
-
       if(fromFormula)
         {
           libmass::Formula newFormula(m_formula);
@@ -461,10 +460,8 @@ MzCalculationDlg::calculate()
                                     isotopic_data_csp,
                                     destIonizeRule.level());
 
-          // qDebug() << __FILE__ << __LINE__
-          //          << "Formula after reionization (level)"
-          //          << level
-          //          << newFormula.text();
+          // qDebug() << "Formula after reionization (level)" << level
+          //          << newFormula.toString();
 
 
           libmass::StringProp *prop = new libmass::StringProp(
@@ -538,7 +535,7 @@ MzCalculationDlg::getDestIonizeRuleData(libmass::IonizeRule *ionizeRule)
 }
 
 
-libmass::Ponderable *
+libmass::Ponderable
 MzCalculationDlg::getSourcePonderable()
 {
   QString text;
@@ -594,8 +591,10 @@ MzCalculationDlg::getSourcePonderable()
         }
     }
 
+  // qDebug() << "We could gather masses:" << mono << "-" << avg;
+
   // At this point we can allocate the Ponderable:
-  return new libmass::Ponderable(mono, avg);
+  return libmass::Ponderable(mono, avg);
 }
 
 
@@ -652,6 +651,12 @@ MzCalculationDlg::prepareResultsTxtString()
     "\nSource conditions:\n"
     "------------------\n");
 
+  if(m_formula.totalAtoms())
+  {
+    *mpa_resultsString +=
+    QObject::tr("Initial formula: %1\n\n").arg(m_formula.toString());
+  }
+
   *mpa_resultsString +=
     QObject::tr("Mono mass: %1").arg(m_ui.monoLineEdit->text());
 
