@@ -32,10 +32,13 @@
 
 
 /////////////////////// Qt includes
-#include <IsotopicDataUserConfigHandler.hpp>
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QCloseEvent>
+
+
+/////////////////////// libmass includes
+#include <libmass/IsotopicDataUserConfigHandler.hpp>
 #include <libmass/PolChemDef.hpp>
 
 
